# Title: StateSoils2
# Description: Use pre-processed SSURGO data for relevant state and extract soils data for basin of interest
# Software Requirements: ArcGIS 10, Spatial Analyst Extension
# Original Authors: Jeremy Hatfield, Robert Coville
# Last Edited by Original Authors: Summer 2018
# Last Edited: 7/8/2020 by James Kruegler
#===============================================================================

import os
import arcinfo, os
import shutil
import csv
from arcpy.sa import *
from arcpy import *
from arcpy.analysis import *
from arcpy.management import *
arcpy.CheckOutExtension("Spatial")
path = os.getcwd()
cpath = path+'\\Constants\\'
soilpath = cpath+'StatesSoils\\'

# ssoilinfo arguments:
#	~ ST: array containing US state abbreviations (strings) for the states intersecting the basin shapefile
#		~ e.g., if the basin intersects New York and New Jersey then ST = ['NY', 'NJ']
#	~ BasinShape: string name of the basin shapefile, without path. Must be located in the basin results directory indicated by 'rpath'
#	~ ID: string of the 8-digit code (including leading zeroes) identifying the current basin
def ssoilinfo(ST,BasinShape,ID):
    # Allow existing files to be overwritten
    env.overwriteOutput = True
    # This dictionary will hold pathnames of all the Buffer-required SSURGO data, for each of the states in ST
    dct = {}
    rpath = path+'\Results'+ID+'\\'
    # Find the geodatabase file created in Buffer
    gdbpath = rpath+'\\results.gdb\\'
    # Define this geodatabase as the current workspace (to match what Buffer does)
    env.workspace = gdbpath
    # When attribute tables are joined together, simplify field names by dropping the substring that is the original table name
    env.qualifiedFieldNames = False
    # Generate array of the state folder names within the \\Constants\\StatesSoils\\ folder
    states = [fol for fol in os.listdir(soilpath) if len(fol) == 2]
    # Looping through the original array of state abbreviations in ST:
    for s in ST:
        # Establish the current state abbreviation as a key in the dct dictionary
        dct.setdefault(s,{})
        # Looping through the array of state folder names in \\StatesSoils\\:
        for state in states:
            # When the code finds a state folder that matches the current state abbreviation:
            if s == str(state):
                # Looping through the files in the matching state folder, established in the line above:
                for fil in os.listdir(os.path.join(soilpath,state)):
                    # Identify the SSURGO geodatabase within the state folder
                    if '.gdb' in fil and str(state) in fil:
                        gdb = os.path.join(soilpath,state,fil)
                        # Search gdb for shapefile and tables
                        # arcpy.da.Walk searches through the SSURGO geodatabase and returns tuples (strictly ordered lists) of path-folder-file combinations
                        # 	p is the full path of gdb as a string
                        # 	d is a list of names of subdirectories and other workspaces in p
                        # 	q is a list of names of nonworkspace contents in p
                        for p, d, q in arcpy.da.Walk(gdb):
                            # Looping through the filenames in each list of files (i.e., each q) generated by arcpy.da.Walk:
                            for fil in q:
                                # Identify the MUPOLYGON feature class, which has basic info about the top level (map unit) of soils data in its attribute table
                                if fil == 'MUPOLYGON':
                                    Shape = os.path.join(gdb,fil)
                                    # Project MUPOLYGON to the same projection as the basin shapefile; result is called 'SoilPoly'
                                    arcpy.Project_management(Shape, gdbpath+'SoilPoly', arcpy.Describe(rpath+BasinShape).spatialReference)
                                    # Clip SoilPoly with the basin shapefile; result is called 'SoilClip'+(state abbreviation)
                                    Clip_analysis(gdbpath+'SoilPoly', rpath+BasinShape, 'SoilClip'+str(s))
                                    # Assign the SoilClip filename to a place within the dct dictionary, under the current state abbreviation key
                                    dct[s]['Shape'] = 'SoilClip'+str(s)
                                # Identify the 'chorizon' geodatabase table, which stores data for horizon-scale parameters
                                if fil == 'chorizon':
                                    # Add the horizon table pathname to the dictionary
                                    dct[s]['Horizon'] = os.path.join(gdb,fil)
                                # Identify the 'component' geodatabase table, which stores data for component-scale parameters (components are made up of horizons)
                                if fil == 'component':
                                    # Add the component table pathname to the dictionary
                                    dct[s]['Component'] = os.path.join(gdb,fil)
                                # Identify the 'muaggatt' geodatabase table, which stores data for map unit-scale parameters (map units are made up of components)
                                if fil == 'muaggatt':
                                    # Add the map unit table pathname to the dictionary
                                    dct[s]['MapUnit'] = os.path.join(gdb,fil)
    # This array will hold filenames of feature layers with all Buffer-required SSURGO tables joined to them
    # Should be one feature layer for each state in ST
    Ssoil=[]
    # Looping through the state abbreviation keys in the dictionary (should be the same list of state abbreviations as ST):
    for st in dct:
        # Create a temporary feature layer out of the SoilClip polygon; result is called 'SoilLyr'+(state abbreviation)
        lyer = MakeFeatureLayer_management(gdbpath+dct[st]['Shape'], gdbpath+'SoilLyr'+str(st), "", '', "")
        # Save a permanent copy of SoilLyr; result is called 'Soil'+(state abbreviation)
        CopyFeatures_management(lyer, gdbpath+'Soil'+str(st))
        # Join the 'muaggatt', 'component', and 'horizon' tables to the 'Soil' feature layer
        # 'muaggatt' and 'component' are joined using the map unit key ('mukey') common field
        muagg = AddJoin_management(lyer, 'MUKEY', dct[st]['MapUnit'], "mukey", "KEEP_ALL")
        comp = AddJoin_management(lyer, 'MUKEY', dct[st]['Component'], "mukey", "KEEP_ALL")
        # 'horizon' is joined using the component key ('cokey') common field
        hrzn = AddJoin_management(lyer, 'component.cokey', dct[st]['Horizon'], "cokey", "KEEP_ALL")
        # Save a permanent copy of the final version of the feature layer (now contained in hrzn); result is called 'merge'+(state abbreviation)
        CopyFeatures_management(hrzn, gdbpath+'merge'+str(st))
        # Add the 'merge' feature layer to the Ssoil array
        Ssoil.append(str(gdbpath+'merge'+st))
        # Delete extra copies of the same SSURGO data
        Delete_management('Soil'+st)
        Delete_management('SoilClip'+st)
    print 'Soil Joins Complete'
    # If there is more than one feature class name in Ssoil (i.e., the basin shapefile intersected >1 state):
    if len(Ssoil)>1:
        # Join together the names as one string
        shps = ';'.join(Ssoil)
        #Merge the feature classes together; result is called 'SoilArea'
        Sshp = Merge_management(Ssoil,"SoilArea",'Join')
    # If there is only one feature class name in Ssoil (i.e., the basin shapefile lies completely within 1 state):
    else:
        # Identify the first (only) feature class in the list
        Sshp = Ssoil[0]
        # Make a new copy of that feature class, called 'SoilArea'
        CopyFeatures_management(Sshp,'SoilArea')


